using DoubleDeckerBus.DrawningObjects;
using DoubleDeckerBus.MovementStrategy;

namespace DoubleDeckerBus
{
    public partial class DoubleDeckerBus : Form
    {
        private DrawningBus? _drawningBus;
        private AbstractStrategy? _strategy;
        public DrawningBus? SelectedBus { get; private set; }
        public DoubleDeckerBus()
        {
            InitializeComponent();
            _strategy = null;
            SelectedBus = null;
        }
        private void Draw()
        {
            if (_drawningBus == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxBus.Width,
            pictureBoxBus.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningBus.DrawTransport(gr);
            pictureBoxBus.Image = bmp;
        }
        private void buttonCreateBus_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningBus = new DrawningBus(random.Next(100, 300),
            random.Next(1000, 3000), color,
            pictureBoxBus.Width, pictureBoxBus.Height);
            _drawningBus.SetPosition(random.Next(10, 100), random.Next(10,
            100));
            Draw();
        }
        private void buttonCreatDoubleDeckerDus_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogColor = new();
            if (dialogColor.ShowDialog() == DialogResult.OK)
            {
                color = dialogColor.Color;
            }
            Color dopColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogDopColor = new();
            if (dialogDopColor.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialogDopColor.Color;
            }
            _drawningBus = new DrawningDoubleDeckerBus(random.Next(100, 300),
            random.Next(1000, 3000), color,
            dopColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxBus.Width, pictureBoxBus.Height);
            _drawningBus.SetPosition(random.Next(10, 100), random.Next(10,
            100));
            Draw();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningBus == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningBus.MoveTransport(DirectionBus.Up);
                    break;
                case "buttonDown":
                    _drawningBus.MoveTransport(DirectionBus.Down);
                    break;
                case "buttonLeft":
                    _drawningBus.MoveTransport(DirectionBus.Left);
                    break;
                case "buttonRight":
                    _drawningBus.MoveTransport(DirectionBus.Right);
                    break;
            }
            Draw();
        }
        private void buttonStep_Click_Click(object sender, EventArgs e)
        {
            if (_drawningBus == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _strategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(_drawningBus.GetMoveableObject,
                pictureBoxBus.Width, pictureBoxBus.Height);
            }
            if (_strategy == null)
            {
                return;
            }
            comboBoxStrategy.Enabled = false;
            _strategy.MakeStep();
            Draw();
            if (_strategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _strategy = null;
            }
        }
        private void buttonSelectBus_Click(object sender, EventArgs e)
        {
            SelectedBus = _drawningBus;
            DialogResult = DialogResult.OK;
        }
    }
}

