﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DoubleDeckerBus.DrawningObjects;
using DoubleDeckerBus.Generics;
using DoubleDeckerBus.MovementStrategy;
using static System.Windows.Forms.DataFormats;
using System.Windows.Forms;
using DoubleDeckerBus.Exceptions;
using Microsoft.Extensions.Logging;
using System.Xml.Linq;



namespace DoubleDeckerBus
{
    public partial class FormBusCollection : Form
    {
        private readonly BusesGenericStorage _storage;
        private readonly ILogger _logger;
        public FormBusCollection(ILogger<FormBusCollection> logger)
        {
            InitializeComponent();
            _storage = new BusesGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;
        }
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;

            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBuses();
        }
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Добавлен набор:{textBoxStorageName.Text}");
        }
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Удалить объект {name}?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(name);
                ReloadObjects();
                _logger.LogInformation($"Удален набор: {name}");
            }
        }
        private void buttonAddBus_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            FormBusConfig form = new();
            form.Show();
            Action<DrawningBus>? busDelegate = new((m) =>
            {
                try
                {
                    bool q = obj + m;
                    MessageBox.Show("Объект добавлен");
                    _logger.LogInformation($"Добавлен объект в коллекцию {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    pictureBoxCollection.Image = obj.ShowBuses();
                }
                catch (StorageOverflowException ex)
                {
                    _logger.LogWarning($"Коллекция {listBoxStorages.SelectedItem.ToString() ?? string.Empty} переполнена");
                    MessageBox.Show(ex.Message);
                }
            });
            form.AddEvent(busDelegate);
        }
        private void buttonRemoveBus_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty]; 
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удалить", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            try
            {
                int pos = Convert.ToInt32(maskedTextBoxNumber.Text); 
                if (obj - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowBuses();
                    _logger.LogInformation($"лодка удалена из набора {listBoxStorages.SelectedItem.ToString()}");
                }
                else
                {
                    MessageBox.Show("Объект не удален");
                    _logger.LogWarning($"лодка не удалена из набора {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (BusNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"BoatNotFound: {ex.Message} in set {listBoxStorages.SelectedItem.ToString()}");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Объект не добавлен"); _logger.LogWarning("Not input");
            }
        }
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBuses();
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно","Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Файл {saveFileDialog.FileName} успешно сохранен");
                }
                catch (Exception ex)
                {
                    _logger.LogWarning("Не удалось сохранить");
                    MessageBox.Show($"Не сохранилось: {ex.Message}","Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }

        }
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно","Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Файл {openFileDialog.FileName} успешно загружен");
                    foreach (var collection in _storage.Keys)
                    {
                        listBoxStorages.Items.Add(collection);
                    }
                    ReloadObjects();
                }
                catch (Exception ex)
                {
                    _logger.LogWarning("Не удалось загрузить");
                    MessageBox.Show($"Не загрузилось: {ex.Message}","Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }
        }
    }
}


